@@ -5,8 +5,9 @@ Functions here can be sped up by JIT, but the code is currently weakly typed.
 In order to use Numba for JIT, they need to be rewritten with clear typing and likely compartmentalized.
 Alternatively, some of the mass functions can be implemented in C and compiled to interface Python.
 '''
-import numpy as np
 # from numba import jit
+
+import numpy as np
 from mass2chem.search import *
 
 
@@ -14,7 +15,6 @@ def flatten_tuplelist(L):
     '''Reformat [(a,b), ...] to [a, b, ...], keep unique entries'''
     return list(set([x[0] for x in L] + [x[1] for x in L]))
 
-
 def check_close_mzs(mzlist, ppm_tol=5):
     '''check potentially overlapping m/z values in a sample. mzlist already in ascending order.'''
     warning = []
@@ -170,6 +170,8 @@ def complete_mass_paired_mapping(list1, list2, std_ppm=5):
     complete_mass_paired_mapping(list1, list2) >>>
     [(0, 2), (3, 6), (6, 8), (10, 23), (12, 24), (14, 28), (16, 32), (23, 43), (25, 49), (26, 60), (29, 65), (31, 66), (36, 70), (38, 71), (41, 72), (44, 77), (46, 81), (51, 85), 
     (52, 89), (53, 91), (1, None), (2, None), (4, None), (5, None), (7, None), ...]
+
+    Update docstring??
     '''
     all = [(list1[ii], 1, ii) for ii in range(len(list1))] + [(list2[jj], 2, jj) for jj in range(len(list2))]
     # [(mz, list_origin, index_origin), ...]
@@ -303,90 +305,47 @@ def landmark_guided_mapping(REF_reference_mzlist, REF_landmarks,
 
 
 
-
-# -----------------------------------------------------------------------------
-#
-# Not used now
 # -----------------------------------------------------------------------------
 
 
-
-from collections import namedtuple
-
-# from scipy.stats import norm as normal_distribution
-
-
-# feature id will be assigned at the end; intensities is a list; mass_id links to MassTrace
-Feature = namedtuple('Feature', ['feature_id', 'mass_id', 'mz', 'rtime', 'rt_min', 'rt_max', 
-                                'peak_quality_max', 'peak_quality_median', 'number_peaks', 'perc_peaks',
-                                'selectivity_combined', 'selectivity_mz', 'intensity_mean',
-                                'intensities'])
-
-def peaks_to_features(peak_dict, rtime_tolerance, ordered_sample_names):
-    
-    '''peak_dict: {formula_mass or _M_id: list of Peaks}
-    return List of Features (namedTuples, 'mass_id,mz,rtime,peak_quality,selectivity_rt,intensities'), 
-    following the input sample order.
+def seed_nn_mz_cluster(bin_data_tuples, seed_indices):
     '''
+    complete NN clustering
+    '''
+    seeds = [bin_data_tuples[ii] for ii in seed_indices]
+    _NN = len(seed_indices)
+    clusters = [[]] * _NN
+    # assing cluster number by nearest distance to a seed
+    for x in bin_data_tuples:
+        deltas = sorted([(abs(x[0]-seeds[ii][0]), ii) for ii in range(_NN)])
+        clusters[deltas[0][1]].append(x)
+
+    return clusters
+
+
+def gap_divide_mz_cluster(bin_data_tuples, mz_tolerance):
+    '''
+    return all clusters and each must be within mz_tolerance
+    '''
+    def __divide_by_largest_gap__(L):
+        gaps = [L[ii][0]-L[ii-1][0] for ii in range(1, len(L))]
+        largest = np.argmax(gaps) + 1
+        return L[:largest], L[largest:]
+
+    def __group_by_tolcheck__(good, bad, mz_tolerance):
+        tmp = []
+        for TT in bad:
+            for C in __divide_by_largest_gap__(TT):
+                # 
+                if C[-1][0] - C[0][0] < mz_tolerance:
+                    good.append(C)
+                else:
+                    tmp.append(C)
+
+        return good, tmp
+
+    good, bad = [], [bin_data_tuples]
+    while bad:
+        good, bad = __group_by_tolcheck__(good, bad, mz_tolerance)
     
-    def __get_peaks_intensities__(peaks, ordered_sample_names):
-        dict_intensities = {}
-        for P in peaks: 
-            if P.sample_name in dict_intensities:
-                dict_intensities[P.sample_name] += P.peak_area      # redundant peaks in same m/z & rt are merged/summed here
-            else:
-                dict_intensities[P.sample_name] = P.peak_area
-        return [dict_intensities.get(name, 0) for name in ordered_sample_names]
-
-    def __bin_by_median_rt__(List_of_peaks, tolerance):
-        List_of_tuples = [(P.cal_rtime, P) for P in List_of_peaks]
-        List_of_tuples.sort()
-        return bin_by_median(List_of_tuples, lambda x: max(tolerance, 0.1*x))
-
-    FeatureList = []            # still possibly not distinguishing close peaks well, which may be treated as combined peaks. 
-    for k,v in peak_dict.items():
-        for F in __bin_by_median_rt__(v, rtime_tolerance):
-            median_mz, median_rt = np.median([P.mz for P in F]), np.median([P.cal_rtime for P in F])
-            feature_id = str(round(median_mz,4)) + '@' + str(round(median_rt,2))
-            rt_min = float(min([P.left_rtime for P in F]))          # checking if this is np.array ???
-            rt_max = float(max([P.right_rtime for P in F]))
-            peak_quality_max = max([P.goodness_fitting for P in F])
-            peak_quality_median = np.median([P.goodness_fitting for P in F])
-            number_peaks = len(set([P.sample_name for P in F]))
-            perc_peaks = 100.0 * number_peaks/len(ordered_sample_names)
-            selectivity_mz = np.mean([P.selectivity for P in F])
-            selectivity_combined = 9
-            intensities = __get_peaks_intensities__(F, ordered_sample_names)
-            intensity_mean = int(sum(intensities)/number_peaks)
-            FeatureList += [Feature(feature_id, k, median_mz, median_rt, rt_min, rt_max, 
-                            peak_quality_max, peak_quality_median, number_peaks, perc_peaks,
-                            selectivity_combined, selectivity_mz, intensity_mean, intensities)]
-
-    return FeatureList
-
-
-# not used now
-def _get_downstair_neighbor_mixedlist_(all_sorted_list, ii, ii_limit=0):
-    # return nearest m/z neighbor of same list_origin in sorted [(mz, list_origin, index_origin), ...]
-    # -1 indicates none found
-    list_origin = all_sorted_list[ii][1]        # from list 1 or 2
-    ii -= 1
-    while ii >= ii_limit and all_sorted_list[ii][1] != list_origin:
-        ii -= 1
-    if all_sorted_list[ii][1] == list_origin:
-        return ii
-    else:
-        return -1
-
-def _get_upstair_neighbor_mixedlist_(all_sorted_list, ii, ii_limit):
-    # return nearest m/z neighbor of same list_origin in sorted [(mz, list_origin, index_origin), ...]
-    # -1 indicates none found
-    list_origin = all_sorted_list[ii][1]        # from list 1 or 2
-    ii += 1
-    while ii < ii_limit and all_sorted_list[ii][1] != list_origin:
-        ii += 1
-    if all_sorted_list[ii][1] == list_origin:
-        return ii
-    else:
-        return -1
-
+    return good
