@@ -1,5 +1,7 @@
 '''
-Functions for peak detection and evaluation.
+Functions for elution peak detection and evaluation.
+stats_detect_elution_peaks and deep_detect_elution_peaks use different supporting functions,
+as the former deals with conversion of RT coordinates.
 '''
 
 import multiprocessing as mp
@@ -10,63 +12,35 @@ from scipy.optimize import curve_fit
 from .chromatograms import *
 
 # -----------------------------------------------------------------------------
-# peak evaluation
+# multicore processing for peak detection
 # -----------------------------------------------------------------------------
 
-def gaussian_function__(x, a, mu, sigma):
-    return a*np.exp(-(x-mu)**2/(2*sigma**2)) 
-
-def goodness_fitting__(y_orignal, y_fitted):                  # R^2 as goodness of fitting
-    return 1 - (np.sum((y_fitted-y_orignal)**2) / np.sum((y_orignal-np.mean(y_orignal))**2))
-
-def evaluate_gaussian_peak(mass_track, peak):
-    '''
-    Use Gaussian models to fit peaks, R^2 as goodness of fitting.
-    mass_track: {'id_number': k, 'mz': mz, 'intensity': [..]}
-    Peak: {'parent_masstrace_id': 2812, 'mz': 359.9761889867791, 'apex': 91.0, 'height': 491241.0, 'left_base': 49.0, 'right_base': 267.0}
-    return: goodness_fitting
-    '''
-    goodness_fitting = 0
-    xx = range(peak['left_base'], peak['right_base']+1, 1)
-    yy = mass_track['intensity'][xx]
-    # yy = mass_track['intensity'][peak['left_index']: peak['right_index']+1]
-    # set initial parameters
-    a, mu, sigma =  peak['height'], peak['apex'], np.std(xx)
-    try:
-        popt, pcov = curve_fit(gaussian_function__, xx, yy, p0=[a, mu, sigma])
-        goodness_fitting = goodness_fitting__( yy, gaussian_function__(xx, *popt))
-    # failure to fit
-    except (RuntimeError, ValueError):
-        # about 50 occurancies on one dataset # print(peak['parent_masstrace_id'], peak['apex'])
-        goodness_fitting = 0
-
-    return goodness_fitting
-
-
-# -----------------------------------------------------------------------------
-# peak detection
-# -----------------------------------------------------------------------------
-
-
 def batch_deep_detect_elution_peaks(list_mass_tracks, max_rt_number, parameters):
     with mp.Manager() as manager:
         shared_list = manager.list()
         iters = iter_peak_detection_parameters(list_mass_tracks, max_rt_number, parameters, shared_list)
         with mp.Pool( parameters['multicores'] ) as pool:
-            pool.starmap( deep_detect_elution_peaks, iters )
+            # switch peak detection algorithms
+            # pool.starmap( deep_detect_elution_peaks, iters )
+            pool.starmap( stats_detect_elution_peaks, iters )
 
         FeatureList = list(shared_list)
     return FeatureList
 
 def iter_peak_detection_parameters(list_mass_tracks, max_rt_number, parameters, shared_list):
+    '''
+    Prominence requirement is critical to peak detection. 
+    This is dynamically determined based on parameters['min_prominence_threshold'] 
+    and intensity level in a region (min_prominence_ratio * __max_intensity).
+    '''
     iters = []
     min_peak_height = parameters['min_peak_height']
     min_prominence_threshold = parameters['min_prominence_threshold']
     min_fwhm = round( 0.5 * parameters['min_timepoints'] )
     snr = parameters['signal_noise_ratio']
-    wlen = 50
+    wlen = 101                          # if used, this limits peak evaluation 50 scans to the left and 50 to right
     min_prominence_ratio = 0.1
-    iteration = False
+    iteration = True                    # a 2nd round of peak detection if enough remaining datapoints
     for mass_track in list_mass_tracks:
         if mass_track['intensity'].max() > min_peak_height:
             iters.append(
@@ -76,37 +50,168 @@ def iter_peak_detection_parameters(list_mass_tracks, max_rt_number, parameters,
     return iters
 
 
+# -----------------------------------------------------------------------------
+# Statistics guided peak detection
+# -----------------------------------------------------------------------------
+
+def stats_detect_elution_peaks(mass_track, max_rt_number, 
+                min_peak_height, min_fwhm, min_prominence_threshold,
+                wlen, snr, min_prominence_ratio, iteration,
+                shared_list):
+    '''
+    Stats guided peak detection. Use statistical summary (stdev) to get ROI (region of interest), 
+    apply smooth_moving_average and determine prominence per ROI.
+    The advantages of separating ROI are 1) better performance of long LC runs, and
+    2) big peaks are less likely to shallow over small peaks. 
+    iteration: if True, a 2nd round of peak detection if enough remaining datapoints.
+
+    No more gap allowed in peaks, since asari is using 
+    composite mass track - gaps should not exist after combining many samples.
+    Now position indices have to be converted internally in this function.
+
+    Input
+    =====
+    mass_track: {'id_number': k, 'mz': mz, 'rt_scan_numbers': [..], 'intensity': [..]}
+                Mass tracks are expected to be continuous per m/z value, with 0s for gaps.
+    snr: minimal signal to noise ratio required for a peak. 
+                Noise is defined by the mean of all non-peak data points in mass_track.
+    min_peak_height, min_fwhm, min_prominence_threshold as in main parameters.
+    min_prominence_ratio: require ratio of prominence relative to peak height.
+    wlen, iteration, are not used in this function but kept for consistent format with others.
+
+    Update
+    ======
+    shared_list: list of peaks in JSON format, to pool with batch_deep_detect_elution_peaks.
+    '''
+    list_json_peaks, list_peaks = [], []
+    list_intensity = __list_intensity = mass_track['intensity']
+    list_scans = np.arange(max_rt_number)
+
+    __number_zeros__ = len(list_scans[list_intensity < 1])
+    if list_intensity.max() > 100*min_peak_height and __number_zeros__ > 0.5*max_rt_number:
+        _do_smoothing = False           # clean mass track
+        __noise_level__ = 1
+    else:
+        _do_smoothing = True            # noisy mass track
+        __noise_level__ = min(list_intensity.std(), min_peak_height * min_prominence_ratio)     
+
+    ROIs = []                       # get ROIs by separation/filtering with __noise_level__
+    __selected_scans__ = list_scans[list_intensity > __noise_level__]
+    tmp = [__selected_scans__[0]]
+    for ii in __selected_scans__[1:]: # get continuous strips
+        if ii == tmp[-1] + 1:
+            tmp.append(ii)
+        else:
+            ROIs.append(tmp)
+            tmp = [ii]
+        
+    ROIs = [r for r in ROIs if len(r) > min_fwhm]
+    for R in ROIs:
+        list_json_peaks += _detect_regional_peaks( list_intensity[R], R, 
+                    min_peak_height, min_fwhm, min_prominence_threshold, min_prominence_ratio, 
+                    _do_smoothing
+        )
+
+    # evaluate and format peaks
+    _peak_datapoints = []
+    list_cSelectivity = __peaks_cSelectivity_stats_(__list_intensity, list_json_peaks)
+    for ii in range(len(list_json_peaks)):
+        list_json_peaks[ii]['cSelectivity'] = list_cSelectivity[ii]
+        _peak_datapoints += list(range(list_json_peaks[ii]['left_base'], list_json_peaks[ii]['right_base']+1))
+
+    _noise_datapoints = [ii for ii in range(max_rt_number) if ii not in _peak_datapoints]
+    __noise_level__ = list_intensity[_noise_datapoints].mean()
+    for peak in list_json_peaks:
+        peak['parent_masstrack_id'] = mass_track['id_number']
+        peak['mz'] = mass_track['mz']
+        peak['snr'] = int( min(peak['height'], 99999999) / __noise_level__)         # cap upper limit and avoid INF
+        peak['goodness_fitting'] = evaluate_gaussian_peak(mass_track, peak)
+        if peak['snr'] > snr:
+            list_peaks.append(peak)
+
+    shared_list += list_peaks
+
+def _detect_regional_peaks(list_intensity_roi, rt_numbers_roi, 
+                    min_peak_height, min_fwhm, min_prominence_threshold, min_prominence_ratio,
+                    smoothing=True):
+    '''
+    Return list of peaks based on detection in ROI, defined by (list_intensity_roi, rt_numbers_roi).
+    smooth_moving_average is applied before peak detection.
+    Smoothing will reduce the height of narrow peaks in CMAP, but not on the reported values,  
+    because peak area is extracted from each sample after. 
+    The likely slight expansion of peak bases adds to robustness.
+    '''
+    list_peaks = []
+    if smoothing:
+        list_intensity_roi = smooth_moving_average(list_intensity_roi, size=9)
+    prominence = max(min_prominence_threshold, min_prominence_ratio * list_intensity_roi.max())
+    peaks, properties = find_peaks(list_intensity_roi, 
+                                    height=min_peak_height, width=min_fwhm, prominence=prominence) 
+
+    for ii in range(peaks.size):
+        _jpeak = convert_roi_peak_json_(ii, list_intensity_roi, rt_numbers_roi, peaks, properties, )
+        list_peaks.append(_jpeak)
+
+    return check_overlap_peaks(list_peaks)
+
+def convert_roi_peak_json_( ii, list_intensity_roi, rt_numbers_roi, peaks, properties):
+    '''
+    peaks, properties as from scipy find_peaks; list_intensity_roi, rt_numbers_roi from mass_track.
+    This handles the conversion btw indices of ROI and indices of mass_track.
+    '''
+    left_index, right_index = properties['left_bases'][ii], properties['right_bases'][ii]   # index positions on ROI
+    left_base, right_base = rt_numbers_roi[left_index], rt_numbers_roi[right_index]
+    peak_area = int( list_intensity_roi[left_index: right_index+1].sum() ) 
+    return {
+            'apex': rt_numbers_roi[peaks[ii]], 
+            'peak_area': peak_area,
+            'height': int(properties['peak_heights'][ii]),
+            'left_base': left_base,
+            'right_base': right_base, 
+    }
+
+def __peaks_cSelectivity_stats_(__list_intensity, _jpeaks):
+    '''
+    peaks, properties as from find_peaks; 
+    __list_intensity is np.array of the mass_track.
+    See also __peaks_cSelectivity__.
+    '''
+    _peak_datapoints = []
+    for peak in _jpeaks:
+        _peak_datapoints += list(range(peak['left_base'], peak['right_base']))
+    _peak_datapoints = __list_intensity[list(set(_peak_datapoints))]    # peaks may overlap
+    list_cSelectivity = []
+    for ii in range(len(_jpeaks)):
+        _threshold = 0.5 * _jpeaks[ii]['height']
+        _peak_datapoints_level = _peak_datapoints[_peak_datapoints > _threshold].size
+        _background_level = __list_intensity[__list_intensity > _threshold].size
+        if _background_level >= _peak_datapoints_level > 0:                  # to rule out artifact from smooth_lowess
+            list_cSelectivity.append(_peak_datapoints_level / _background_level)
+        else:
+            list_cSelectivity.append( 0 )
+            
+    return list_cSelectivity
 
 
+# -----------------------------------------------------------------------------
+# Heuristics peak detection
+# -----------------------------------------------------------------------------
 
-
-
-
-def deep_detect_elution_peaks( mass_track, max_rt_number, 
+def deep_detect_elution_peaks(mass_track, max_rt_number, 
                 min_peak_height, min_fwhm, min_prominence_threshold,
                 wlen, snr, min_prominence_ratio, iteration,
                 shared_list):
     '''
     Optimized peak detection on a mass track. 
-
-    min_peak_height=10000, min_fwhm=3, min_prominence_threshold=5000, wlen=50, 
-                snr=2, min_prominence_ratio=0.1,
-                iteration=True, 
-
-    Can add further stats based ROI selection to optimize -
-                - not requiring continuous track, but ROIs will be continuous and used for peak detection 
-
-
-
     Local maxima is used in find_peak, but prominence and cSelectivity are used to control detection,
     in step-wise optimization. 
     Peak shape (Gaussian fitting goodness > 0.8) is required for noisy data or iterated detection.
-    Noisy tracks are smoothed using moving average of 3 data points.
+    Noisy tracks are smoothed using moving average of N data points.
     peak area is integrated by summing up intensities of included scans.
     Reported left/right bases are not based on Gaussian shape or similar, just local maxima.
     Noise level is defined as mean of all non-peak, nonzero data points.
 
-    Iterations are only done when good peaks are detected in the first round, 
+    Iterations should only be done when good peaks are detected in the first round, 
     to ensure no small peaks are skipped due to prominence control in round 1,
     by removing round 1 peaks from the track in the next peak detection.
     Iterations are not performed on noisy data tracks.
@@ -126,8 +231,8 @@ def deep_detect_elution_peaks( mass_track, max_rt_number,
                                                 'goodness_fitting', 'snr', 'cSelectivity',}, ...]
     '''
     list_json_peaks, list_peaks = [], []
-    list_intensity = mass_track['intensity']
-    __list_intensity = np.array(list_intensity)
+    list_intensity = __list_intensity = mass_track['intensity']   # np.array(list_intensity)
+    
     __max_intensity = __list_intensity.max()
     __exam_window_size = 0.1*max_rt_number
     prominence = max(min_prominence_threshold, 
@@ -140,10 +245,6 @@ def deep_detect_elution_peaks( mass_track, max_rt_number,
         new_list_intensity = list_intensity.copy()
         if peaks.size <= 2:
             for ii in range(peaks.size):
-
-
-
-
                 _jpeak = convert_peak_json__(ii, mass_track, peaks, properties, cSelectivity=list_cSelectivity[ii])
                 # evaluate peak quality by goodness_fitting of Gaussian model
                 _jpeak['goodness_fitting'] = evaluate_gaussian_peak(mass_track, _jpeak)
@@ -214,7 +315,7 @@ def deep_detect_elution_peaks( mass_track, max_rt_number,
                         if _jpeak['goodness_fitting'] > 0.6:    # requiring good peaks after iteration or in noisy data
                             list_json_peaks.append(_jpeak)
 
-        # do SNR, defined as mean of all non-peak data points
+        # do SNR, noise defined as mean of all non-peak data points
         _peak_datapoints = []
         for _jpeak in list_json_peaks:
             _peak_datapoints += list(range( _jpeak['left_base'], _jpeak['right_base'] ))
@@ -230,13 +331,13 @@ def deep_detect_elution_peaks( mass_track, max_rt_number,
             if _jpeak['snr'] > snr:
                 list_peaks.append(_jpeak)
 
-    shared_list += list_peaks
-
-
+    shared_list += check_overlap_peaks(list_peaks)
 
 def convert_peak_json__( ii, mass_track, peaks, properties, cSelectivity=None):
-    '''peaks, properties as from find_peaks; rt_numbers from mass_track
-
+    '''
+    peaks, properties as from scipy find_peaks; rt_numbers from mass_track
+    When asari switched from mass traces to mass tracks, 
+    the left and right indices no longer needed converstion btw local indices and full RT indices.
     '''
     #  rt_numbers  = mass_track['rt_scan_numbers']
     left_index, right_index = properties['left_bases'][ii], properties['right_bases'][ii]   # index positions on mass track
@@ -249,7 +350,7 @@ def convert_peak_json__( ii, mass_track, peaks, properties, cSelectivity=None):
             # 'rtime': rt_numbers[peaks[ii]], 
             'peak_area': peak_area,
             'height': int(properties['peak_heights'][ii]),
-            'left_base': left_index, # left_base,                                                         # rt_numbers
+            'left_base': left_index, # left_base,                                            # rt_numbers
             'right_base': right_index, # right_base,   
             #'left_index': left_index,                                                       # specific to the referred mass track
             #'right_index': right_index,
@@ -285,10 +386,45 @@ def __peaks_cSelectivity__(__list_intensity, peaks, properties):
     return list_cSelectivity
 
 
+# -----------------------------------------------------------------------------
+# Generic functions for detection and evaluation
+# -----------------------------------------------------------------------------
+
+def gaussian_function__(x, a, mu, sigma):
+    return a*np.exp(-(x-mu)**2/(2*sigma**2)) 
+
+def goodness_fitting__(y_orignal, y_fitted):                  # R^2 as goodness of fitting
+    return 1 - (np.sum((y_fitted-y_orignal)**2) / np.sum((y_orignal-np.mean(y_orignal))**2))
+
+def evaluate_gaussian_peak(mass_track, peak):
+    '''
+    Use Gaussian models to fit peaks, R^2 as goodness of fitting.
+    Peak shapes may be more Voigt or bigaussian, but the impact on fitting result is negligible.
+    mass_track: {'id_number': k, 'mz': mz, 'intensity': [..]}
+    Peak: {'parent_masstrace_id': 2812, 'mz': 359.9761889867791, 'apex': 91.0, 'height': 491241.0, 'left_base': 49.0, 'right_base': 267.0}
+    return: goodness_fitting
+    '''
+    goodness_fitting = 0
+    xx = range(peak['left_base'], peak['right_base']+1, 1)
+    yy = mass_track['intensity'][xx]
+    # yy = mass_track['intensity'][peak['left_index']: peak['right_index']+1]
+    # set initial parameters
+    a, mu, sigma =  peak['height'], peak['apex'], np.std(xx)
+    try:
+        popt, pcov = curve_fit(gaussian_function__, xx, yy, p0=[a, mu, sigma])
+        goodness_fitting = goodness_fitting__( yy, gaussian_function__(xx, *popt))
+    # failure to fit
+    except (RuntimeError, ValueError):
+        # about 50 occurancies on one dataset # print(peak['parent_masstrace_id'], peak['apex'])
+        goodness_fitting = 0
+
+    return goodness_fitting
+
 def detect_elution_peaks( mass_track, 
             min_peak_height=10000, min_fwhm=3, min_prominence_threshold=5000, wlen=50 ):
-    '''Standard peak detection, to be used for exploration.
-    Default in asari is the deep_detect_elution_peaks function.
+    '''
+    Vanila peak detection, to be used for custom processes or exploration.
+    Refer to stats_detect_elution_peaks or deep_detect_elution_peaks for regular functions.
     '''
     list_intensity = mass_track['intensity']
     peaks, properties = find_peaks(list_intensity, height=min_peak_height, width=min_fwhm, 
@@ -302,12 +438,11 @@ def detect_elution_peaks( mass_track,
 
     return list_peaks
 
-
 def quick_detect_unique_elution_peak(intensity_track, 
                             min_peak_height=100000, min_fwhm=3, min_prominence_threshold_ratio=0.2):
-    '''Quick peak detection, only looking for highest peak with high prominence.
-    This is used for quick check on good peaks, or selecting landmarks for alignment purposes.
-    
+    '''
+    Quick peak detection, only looking for a high peak with high prominence.
+    This can be used for quick check on good peaks, or selecting landmarks for alignment purposes.
     '''
     max_intensity = intensity_track.max()
     prominence = min_prominence_threshold_ratio * max_intensity
@@ -322,25 +457,53 @@ def quick_detect_unique_elution_peak(intensity_track,
             }
     return unique_peak
 
-
-
-# -----------------------------------------------------------------------------
-
-def quick_detect_unique_elution_peak_old(rt_numbers, list_intensity, 
-                            min_peak_height=100000, min_fwhm=3, min_prominence_threshold_ratio=0.2):
-    '''Quick peak detection, only looking for highest peak with high prominence.
-    This is used for quick check on good peaks, or selecting landmarks for alignment purposes.
-    rt_numbers, list_intensity are matched vectors from a mass trace/track.
+def check_overlap_peaks(list_peaks):
     '''
-    max_intensity = max(list_intensity)
-    prominence = min_prominence_threshold_ratio * max_intensity
-    unique_peak = None
-    if max_intensity > min_peak_height:
-        peaks, properties = find_peaks(list_intensity, height=min_peak_height, width=min_fwhm, 
-                                                        prominence=prominence) 
-        if peaks.size == 1:
-            unique_peak = {
-                'apex': rt_numbers[peaks[0]], 
-                'height': properties['peak_heights'][0], # not used now
-            }
-    return unique_peak
+    Check overlap btw a list of JSON peaks. 
+    list_peaks are already order by RT from find_peaks. Overlap usually from splitting.
+    Return unique peaks.
+    '''
+    if len(list_peaks) < 2:
+        return list_peaks
+    else:
+        clusters = []
+        tmp = [list_peaks[0]]
+        for peak in list_peaks[1:]:
+            if _check_overlap(peak, tmp[-1]):
+                tmp.append(peak)
+            else:
+                clusters.append(tmp)
+                tmp = [peak]
+        clusters.append(tmp)
+        new = []
+        for C in clusters:
+            new.append(_merge_peak_cluster(C))
+        return new
+
+def _check_overlap(peak1, peak2):
+    '''
+    Check if overlap btw (peak1, peak2) exceeds 3 scans.
+    '''
+    tuple1, tuple2 = (peak1['left_base'], peak1['right_base']), (peak2['left_base'], peak2['right_base'])
+    left_peak, right_peak = tuple1, tuple2
+    if tuple1[1] > tuple2[1]:
+        left_peak, right_peak = tuple2, tuple1
+    overlap = max(0, left_peak[1] - right_peak[0])
+    if overlap > 3: 
+        return True
+    else:
+        return False
+
+def _merge_peak_cluster(cluster_peaks):
+    '''
+    Return: merged peaks by extending bases, inheritting other attributes from largest peak.
+    '''
+    if len(cluster_peaks) == 1:
+        return cluster_peaks[0]
+    else:
+        peak_sizes = [peak['right_base'] - peak['left_base'] for peak in cluster_peaks]
+        largest = np.argmax(peak_sizes)
+        largest = cluster_peaks[largest]
+        largest['left_base'] = min([peak['left_base'] for peak in cluster_peaks])
+        largest['right_base'] = max([peak['right_base'] for peak in cluster_peaks])
+        return largest
